const { app, BrowserWindow, ipcMain } = require("electron");
const path = require("path");

if (process.env.NODE_ENV === "dev") {
  require("electron-reload")(__dirname, {
    electron: path.join(__dirname, "node_modules", ".bin", "electron")
  });
}

function createWindow () {
  // Create the browser window.
  const win = new BrowserWindow({
    width: 640,
    height: 320,
    backgroundColor: "#111",
    resizable: false,
    frame: false,
    webPreferences: {
      nodeIntegration: true
    }
  });

  ipcMain.on("resize", (_event, window) => {
    win.setSize( window.w, window.h);
  });

  // Remove alt menu
  win.removeMenu();

  // and load the index.html of the app.
  win.loadFile(path.join("html/game.html"));

  // Open the DevTools.
  if (process.env.NODE_ENV === "dev") win.webContents.openDevTools();
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.whenReady().then(createWindow);

// Quit when all windows are closed.
app.on("window-all-closed", () => {
  // On macOS it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== "darwin") {
    app.quit();
  }
});

app.on("activate", () => {
  // On macOS it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (BrowserWindow.getAllWindows().length === 0) {
    createWindow();
  }
});

// In this file you can include the rest of your app's specific main process
// code. You can also put them in separate files and require them here.
